# -*- coding: utf-8 -*-
import doctest

from zope.configuration import xmlconfig

from plone.testing import z2

from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.app.testing import login
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing.layers import FunctionalTesting
from plone.app.testing.layers import IntegrationTesting


class PloneAppWidgetsLayer(PloneSandboxLayer):

    def setUpZope(self, app, configurationContext):
        import plone.app.widgets
        xmlconfig.file(
            'configure.zcml',
            plone.app.widgets,
            context=configurationContext)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.app.widgets:default')
        portal.acl_users.userFolderAddUser('admin',
                                           'secret',
                                           ['Manager'],
                                           [])
        login(portal, 'admin')
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")
        setRoles(portal, TEST_USER_ID, ['Manager'])


PLONEAPPWIDGETS_FIXTURE = PloneAppWidgetsLayer()

PLONEAPPWIDGETS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONEAPPWIDGETS_FIXTURE,),
    name="PloneAppWidgetsLayer:Integration")
PLONEAPPWIDGETS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONEAPPWIDGETS_FIXTURE,),
    name="PloneAppWidgetsLayer:Functional")
PLONEAPPWIDGETS_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(PLONEAPPWIDGETS_FIXTURE, z2.ZSERVER_FIXTURE),
    name="PloneAppWidgetsLayer:Acceptance")

optionflags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)



from plone.app.widgets.at.base import PatternsWidget


class DateTimeWidget(PatternsWidget):
    _properties = PatternsWidget._properties.copy()
    _properties.update({
        'pattern': 'datetime',
        'pattern_options': 'formatSubmit:yyyy-m-d H:M;',
    })

    def formatAccessor(self, value, context, field, request):
        if value:
            return value.strftime('%Y-%m-%d %H:%M')
        return value



import json
from zope.component import queryUtility
from zope.component import getMultiAdapter
from zope.schema.interfaces import IVocabularyFactory
from plone.app.widgets.at.base import PatternsWidget


class Select2Widget(PatternsWidget):
    _properties = PatternsWidget._properties.copy()
    _properties.update({
        'pattern': 'select2',
        'pattern_options': '',
        'tags': '',
        'multiple': False,
        'ajax_vocabulary': '',
        'ajax_url': '/@@widgets/getVocabulary?name=',
    })

    def formatAccessor(self, value, context, field, request):
        if type(value) in [list, tuple]:
            return ','.join(value)
        return value

    def updateOptions(self, value, context, field, request):
        state = getMultiAdapter((context, request), name=u'plone_portal_state')
        if self.ajax_vocabulary and self.element_type is not "select":
            self.pattern_options += 'ajaxUrl:' + \
                state.portal_url() + self.ajax_url + self.ajax_vocabulary + ';'
        if self.tags:
            factory = queryUtility(IVocabularyFactory, self.tags)
            tags = [i.token for i in factory(context)]
            self.pattern_options += 'tags:' + json.dumps(tags) + ';'
        if self.multiple is True:
            self.pattern_options += 'multiple:true;'
        if self.ajax_vocabulary:
            factory = queryUtility(IVocabularyFactory, self.ajax_vocabulary)
            vocabulary = factory(context)
            data = {}
            if type(value) in [list, tuple]:
                for item in value:
                    term = vocabulary.getTerm(item)
                    if term:
                        data[term.token] = term.title
                    else:
                        data[item] = item
            else:
                term = vocabulary.getTerm(value)
                data.append(dict(id=term.token, text=term.title))
            self.pattern_options += 'initSelection:' + json.dumps(data) + ';'

    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False, validating=True):
        value = form.get(field.getName(), empty_marker)
        if value is empty_marker:
            return empty_marker
        if emptyReturnsMarker and value == '':
            return empty_marker

        value = value.strip()
        if self.multiple or self.tags:
            value = value.split(',')
        return value, {}



from zope.interface import Interface


class IWidgetsLayer(Interface):
    """Browser layer used to indicate that plone.app.widgets is installed
    """


class IWidgetsView(Interface):
    """A view that gives access to various widget related functions.
    """

    def getVocabulary():
        """Returns vocabulary
        """

    def bodyDataOptions():
        """Returns the data attributes to be used on the body tag.
        """



from zope.component import adapts
from zope.interface import implements
from zope.i18nmessageid import MessageFactory
from Products.ATContentTypes.interface import IATContentType
from archetypes.schemaextender.interfaces import ISchemaModifier
from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender

from plone.app.widgets.interfaces import IWidgetsLayer
from plone.app.widgets.at.select2 import Select2Widget
from plone.app.widgets.at.datetime import DateTimeWidget

_plone = MessageFactory('plone')


class ATWidgetsExtender(object):
    """
    """

    implements(ISchemaModifier, IBrowserLayerAwareExtender)
    adapts(IATContentType)
    layer = IWidgetsLayer

    def __init__(self, context):
        self.context = context

    def fiddle(self, schema):
        for field in schema.fields():
            old = field.widget

            if field.__name__ in ['subject']:
                field.widget = Select2Widget(
                    label=old.label,
                    description=old.description,
                    pattern_options="width:30em;",
                    tags='plone.app.vocabularies.Keywords',
                )

            if field.__name__ in ['language']:

                field.widget = Select2Widget(
                    label=old.label,
                    description=old.description,
                    pattern_options='width:15em;',
                    element_type="select",
                )

            if field.__name__ in ['effectiveDate', 'expirationDate']:
                field.widget = DateTimeWidget(
                    label=old.label,
                    description=old.description
                )

            if field.__name__ in ['contributors']:
                field.widget = Select2Widget(
                    label=old.label,
                    description=_plone(u"The names of people that have "
                                       u"contributed to this item."),
                    multiple=True,
                    ajax_vocabulary="plone.app.vocabularies.Users",
                    pattern_options="width:30em;placeholder:Add "
                                    "contributors...;",
                )

            if field.__name__ in ['creators']:
                field.widget = Select2Widget(
                    label=old.label,
                    description=_plone(u"The names of people that have "
                                       u"contributed to this item."),
                    multiple=True,
                    ajax_vocabulary="plone.app.vocabularies.Users",
                    pattern_options="width:30em;placeholder:Add creators...;"
                )

        #for fieldname in ['contributors', 'creators']:
        #    if fieldname in schema:
        #        field = schema[fieldname]
        #        widget = field.widget
        #        field.widget = AutocompleteWidget(
        #            label=widget.label,
        #            description=widget.description,
        #        )
        #if 'customViewFields' in schema:
        #    field = schema['customViewFields']
        #    widget = field.widget
        #    field.widget = ChosenWidget(
        #        label=widget.label,
        #        description=widget.description,
        #        js_options={
        #            'allow_sortable': True
        #        }
        #    )

        #if 'relatedItems' in schema:
        #    field = schema['relatedItems']
        #    widget = field.widget
        #    field.widget = ChosenAjaxWidget(
        #        label=widget.label,
        #        description=widget.description,
        #        ajax_rel_url='widget-catalog-query'
        #    )



import json
import inspect
from zope.interface import implements
from zope.component import queryUtility
from zope.component import getMultiAdapter
from zope.schema.interfaces import IVocabularyFactory
from Products.Five import BrowserView
from plone.app.widgets.interfaces import IWidgetsView


class WidgetsView(BrowserView):
    """A view that gives access to various widget related functions.
    """

    implements(IWidgetsView)

    def getVocabulary(self):
        """
        """
        self.request.response.setHeader("Content-type", "application/json")

        factory_name = self.request.get('name', None)
        if not factory_name:
            return json.dumps({'error': 'No factory provided.'})

        factory = queryUtility(IVocabularyFactory, factory_name)
        if not factory:
            return json.dumps({
                'error': 'No factory with name "%s" exists.' % factory_name})

        # check if factory excepts query argument
        query = self.request.get('query', '')
        factory_spec = inspect.getargspec(factory.__call__)
        if query and len(factory_spec.args) >= 3 and \
                factory_spec.args[2] == 'query':
            vocabulary = factory(self.context, query)
        else:
            vocabulary = factory(self.context)

        items = []
        for item in vocabulary:
            items.append({'id': item.token, 'text': item.title})

        # TODO: add option for limiting number of results
        # TODO: add option for batching
        # TODO: add option for sorting

        return json.dumps({'results': items})

    def bodyDataOptions(self):
        portal_state = getMultiAdapter(
            (self.context, self.request), name=u'plone_portal_state')
        return {
            'data-portal-navigation-url': portal_state.navigation_root_url(),
            'data-portal-url': portal_state.portal_url(),
            'data-context-url': self.context.absolute_url(),
            'data-pattern': 'plone-tabs',
        }



# -*- coding: utf-8 -*-
try:
    import unittest2 as unittest
except:
    import unittest
    assert unittest

from plone.app.widgets.testing import PLONEAPPWIDGETS_INTEGRATION_TESTING
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, setRoles, login


class PloneAppWidgetsClassTest(unittest.TestCase):

    layer = PLONEAPPWIDGETS_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory('Folder', 'test-folder')
        self.folder = self.portal['test-folder']
        self.types = self.portal.portal_types

    def test_css_registered(self):
        css = getattr(self.portal, 'portal_css')
        css_ids = css.getResourceIds()
        self.assertTrue('++resource++plone.app.widgets.css' in css_ids)

    def test_js_registered(self):
        js = getattr(self.portal, 'portal_javascripts')
        js_ids = js.getResourceIds()
        self.assertTrue('++resource++plone.app.widgets.js' in js_ids)
        import pdb; pdb.set_trace()

    def test_widgets_bundle_in_sunburst_theme(self):
        import pdb; pdb.set_trace()

    def test_skin_path_in_sunburt_theme(self):
        import pdb; pdb.set_trace()

    def test_browser_layer(self):
        import pdb; pdb.set_trace()



from lxml import etree
from Products.Archetypes.Widget import TypesWidget


class PatternsWidget(TypesWidget):
    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro': "patterns_widgets",
        'element_type': 'input',
        'pattern': '',
        'pattern_options': ''
    })

    def view(self, context, field, request):
        return field.getAccessor(context)()

    def edit(self, context, field, request):
        value = field.getAccessor(context)()
        if value is None:
            value = ''

        if hasattr(self, 'updateOptions'):
            self.updateOptions(value, context, field, request)

        formatted = value
        if hasattr(self, 'formatAccessor'):
            formatted = self.formatAccessor(value, context, field, request)

        el = etree.Element(self.element_type)
        el.attrib['name'] = field.getName()

        if self.element_type == 'input':
            el.attrib['type'] = 'text'
            el.attrib['value'] = formatted
        elif self.element_type == 'textarea':
            el.text = formatted
        elif self.element_type == 'select' and field.vocabulary:
            for token, title in field.Vocabulary(context).items():
                option = etree.Element('option')
                option.attrib['value'] = token
                if token == value:
                    option.attrib['selected'] = 'selected'
                option.text = title
                el.append(option)

        else:
            raise Exception(
                "Wrong 'element_type' selected ('%s')." % (self.element_type))

        if self.pattern:
            el.attrib['class'] = 'pat-' + self.pattern
            el.attrib['data-pat-' + self.pattern] = self.pattern_options

        return etree.tostring(el)



from setuptools import setup, find_packages

version = '0.1'

setup(
    name='plone.app.widgets',
    version=version,
    description="better plone widgets",
    long_description='%s\n%s' % (
        open("README.rst").read(),
        open("CHANGES.rst").read(),
    ),
    classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
    ],
    keywords='plone widgets z3cform archetypes',
    author='Nathan Van Gheem',
    author_email='vangheem@gmail.com',
    url='https://github.com/plone/plone.app.widgets',
    license='GPL',
    packages=find_packages(),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        # needed because we use bundles
        'Products.ResourceRegistries>=2.1',
        # nedded because users vocabulary was added here
        'plone.app.vocabularies>=2.1.10',
        # needed for pickadate javascript
        'plone.app.jquery>=1.8.0',
        # needed so it works with jquery >= 1.8
        'plone.app.search>=1.1.2',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
        ],
        'archetypes': [
            'archetypes.schemaextender',
        ],
    },
)
